//#define DIRECT

struct {
  const int n;
  int size;
  GdkColor colors[2];
	/* a pixmap for drawing not directly on a widget */
  GdkPixmap *pixmap;
} board = {8};

struct {
  const float ratio;
  int x;
  int y;
  int size;
  GdkColor color;
} man = {(float)3/4};

struct {
  int dragging;
  int x_old;
  int y_old;
} drag_data;

void draw_board (GdkDrawable *drawable, GdkGC *gc)
{
  int place_size;
  int i = 0, j = 0, c = 0;

  place_size = board.size / board.n;

  while (i < board.size) {
    j = 0;
    while (j < board.size) {
		/* set GC color */
      gdk_gc_set_foreground (gc, &board.colors[c]);
	  /* draw rectangle */
      gdk_draw_rectangle (drawable, gc, TRUE, j, i, place_size, place_size);

      c = !c;
      j += place_size;
    }
    c = !c;
    i += place_size;
  }
}

void draw_man (GdkDrawable *drawable, GdkGC *gc)
{
  gdk_gc_set_foreground (gc, &man.color);
  /* draw full circle */
  gdk_draw_arc (drawable, gc, TRUE, man.x, man.y, man.size, man.size,
		0, 360 * 64);
}

void draw (GtkWidget *widget)
{
  GdkDrawable *drawable;
  GdkGC *gc;

  /* select the widget which is paint on */
#ifdef DIRECT
  drawable = widget->window;
#else
  drawable = board.pixmap;
#endif
  /* get widget's Graphic Context */
  gc = widget->style->fg_gc[GTK_WIDGET_STATE (widget)];

  draw_board (drawable, gc);
  draw_man (drawable, gc);

#ifndef DIRECT
  /* queue redraw */
  gtk_widget_queue_draw (widget);
#endif
}

/* realize event handler: initialize the widget */
void on_drawingarea_realize (GtkWidget *widget, gpointer user_data)
{
  GdkGC *gc;

  board.colors[0].red = board.colors[0].green = board.colors[0].blue = 0;
  board.colors[1].red = board.colors[1].green = board.colors[1].blue = 0xFFFF;
  man.color.red = man.color.green = 0; man.color.blue = 0xFFFF;
  man.x = 50; man.y = 50;

  gc = widget->style->fg_gc[GTK_WIDGET_STATE (widget)];

  /* allocate color */
  gdk_colormap_alloc_color (gdk_gc_get_colormap (gc),
			    &board.colors[0], TRUE, TRUE);
  gdk_colormap_alloc_color (gdk_gc_get_colormap (gc),
			    &board.colors[1], TRUE, TRUE);
  gdk_colormap_alloc_color (gdk_gc_get_colormap (gc),
			    &man.color, TRUE, TRUE);

  //draw (widget);
}

/* configure event handler: emitted when the widget's size changes */
gboolean on_drawingarea_configure_event (GtkWidget *widget,
					 GdkEventConfigure *event,
					 gpointer user_data)
{
  int place_size;

  board.size = (event->width <= event->height) ? event->width : event->height;
  place_size = board.size / board.n;
  board.size = place_size * board.n;
  man.size = (int)(man.ratio * place_size);

  if (board.size) {
#ifndef DIRECT
    if (board.pixmap)
      g_object_unref (board.pixmap);

	/* new size of the pixmap */
    board.pixmap = gdk_pixmap_new (widget->window, board.size, board.size, -1);
#endif

    draw (widget);
  }

  return FALSE;
}

/* expose event handler: repaint a (part of the) widget */
gboolean on_drawingarea_expose_event (GtkWidget *widget,
				      GdkEventExpose *event,
				      gpointer user_data)
{
#ifdef DIRECT
  draw (widget);
#else
  /* draw a part of the pixmap onto the widget */
  gdk_draw_drawable (widget->window,
		     widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		     board.pixmap,
		     event->area.x, event->area.y,
		     event->area.x, event->area.y,
		     event->area.width, event->area.height);
#endif

  return FALSE;
}

/* mouse button press event handler */
gboolean on_drawingarea_button_press_event (GtkWidget *widget,
					    GdkEventButton *event,
					    gpointer user_data)
{
  int x, y;

  /* left mouse button pressed */
  if (event->button == 1) {
    x = (int)event->x; y = (int)event->y;

    if ((x >= man.x) && (x <= (man.x + man.size)) &&
	(y >= man.y) && (y <= (man.y + man.size))) {
      drag_data.dragging = 1;

      drag_data.x_old = x;
      drag_data.y_old = y;
    }
    else
      drag_data.dragging = 0;
  }

  return TRUE;
}

/* mouse motion event handler */
gboolean on_drawingarea_motion_notify_event (GtkWidget *widget,
					     GdkEventMotion *event,
					     gpointer user_data)
{
  int x, y;

  /* left mouse button held */
  if ((event->state & GDK_BUTTON1_MASK) && drag_data.dragging) {
    x = event->x; y = event->y;

    if ((man.x >= 0) && (man.y >= 0) &&
	((man.x + man.size) <= board.size) &&
	((man.y + man.size) <= board.size)) {
      man.x += x - drag_data.x_old;
      man.y += y - drag_data.y_old;

      if ((man.x < 0) || (man.y < 0) ||
	  ((man.x + man.size) > board.size) ||
	  ((man.y + man.size) > board.size)) {
	man.x -= x - drag_data.x_old;
	man.y -= y - drag_data.y_old;
      } else {
	drag_data.x_old = x;
	drag_data.y_old = y;
      }
    }

    draw (widget);

    gdk_event_request_motions (event);
  }

  return TRUE;
}
