//#define DIRECT

#include "area.h"
#include <QtGui>

Area::Area(QWidget *parent)
	: QWidget(parent)
{
	// allocate colors
	board.colors[0] = QColor("black");
	board.colors[1] = QColor("white");
	man.color = QColor("blue");
	man.x = 50; man.y = 50;

	setMinimumSize (500, 500);
	show();
}

void Area::draw_board (QPainter *painter)
{
	int place_size;
	int i = 0, j = 0, c = 0;

	place_size = board.size / board.n;

	// turn off pen
	painter->setPen(Qt::NoPen);
	while (i < board.size) {
		j = 0;
		while (j < board.size) {
			// set brush color
			painter->setBrush(board.colors[c]);
			// draw rectangle
			painter->drawRect(j, i, place_size, place_size);

			c = !c;
			j += place_size;
		}
		c = !c;
		i += place_size;
	}
}

void Area::draw_man (QPainter *painter)
{
	painter->setPen(Qt::NoPen);
	painter->setBrush(man.color);
	// draw full circle
	painter->drawPie(man.x, man.y, man.size, man.size, 0, 360 * 16);
}

void Area::draw ()
{
	// a widget for painting
	QPainter painter;

	// select the widget which is paint on
#ifdef DIRECT
	painter.begin(this);
#else
	painter.begin(&board.pixmap);
#endif

	draw_board (&painter);
	draw_man (&painter);

#ifndef DIRECT
	// queue redraw
	update();
#endif
}

void Area::resizeEvent(QResizeEvent *event)
{
	int place_size;

	board.size = (event->size().width() <= event->size().height()) ?
		event->size().width() : event->size().height();
	place_size = board.size / board.n;
	board.size = place_size * board.n;
	man.size = (int)(man.ratio * place_size);

	if (board.size) {
#ifndef DIRECT
		// new size of the pixmap
		board.pixmap = board.pixmap.copy(0, 0, board.size, board.size);
#endif

		draw();
	}
	QWidget::resizeEvent(event);
}

void Area::paintEvent(QPaintEvent *event)
{
#ifdef DIRECT
	draw ();
#else
	// draw a part of the pixmap onto the widget
	QPainter(this).drawPixmap(event->rect().x(), event->rect().y(),
							  board.pixmap,
							  event->rect().x(), event->rect().y(),
							  event->rect().width(), event->rect().height());
#endif

	QWidget::paintEvent(event);
}

void Area::mousePressEvent(QMouseEvent *event)
{
	int x, y;

	// left mouse button pressed
	if (event->button() == Qt::LeftButton) {
		x = event->x(); y = event->y();

		if ((x >= man.x) && (x <= (man.x + man.size)) &&
			(y >= man.y) && (y <= (man.y + man.size))) {
			drag_data.dragging = 1;

			drag_data.x_old = x;
			drag_data.y_old = y;
		}
		else
			drag_data.dragging = 0;
	}
}

void Area::mouseMoveEvent(QMouseEvent *event)
{
	int x, y;

	// left mouse button held
	if ((event->buttons() & Qt::LeftButton) && drag_data.dragging) {
		x = event->x(); y = event->y();

		if ((man.x >= 0) && (man.y >= 0) &&
			((man.x + man.size) <= board.size) &&
			((man.y + man.size) <= board.size)) {
			man.x += x - drag_data.x_old;
			man.y += y - drag_data.y_old;

			if ((man.x < 0) || (man.y < 0) ||
				((man.x + man.size) > board.size) ||
				((man.y + man.size) > board.size)) {
				man.x -= x - drag_data.x_old;
				man.y -= y - drag_data.y_old;
			} else {
				drag_data.x_old = x;
				drag_data.y_old = y;
			}
		}

		draw ();
	}
}
